@@ -5,8 +5,10 @@
  Portability: portable 
 
 
- This module is for instantiating cryptographically strong determinitic random bit generators (DRBGs, aka PRNGs)
- For the simple use case of using the system random number generator ('System.Crypto.Random') to seed the DRBG:
+ This module is for instantiating cryptographically strong
+determinitic random bit generators (DRBGs, aka PRNGs) For the simple
+use case of using the system random number generator
+('System.Crypto.Random') to seed the DRBG:
 
 @   g <- newGenIO
 @
@@ -54,33 +56,42 @@ data GenError =
 	| NeedsInfiniteSeed	-- ^ This generator can not be instantiated or reseeded with a finite seed (ex: 'SystemRandom')
   deriving (Eq, Ord, Show)
 
--- |A class of random bit generators that allows for the possibility of failure,
--- reseeding, providing entropy at the same time as requesting bytes
+-- |A class of random bit generators that allows for the possibility
+-- of failure, reseeding, providing entropy at the same time as
+-- requesting bytes
 --
--- Minimum complete definition: `newGen`, `genSeedLength`, `genBytes`, `reseed`.
+-- Minimum complete definition: `newGen`, `genSeedLength`, `genBytes`,
+-- `reseed`.
 class CryptoRandomGen g where
-	-- |Instantiate a new random bit generator.  The provided bytestring should
-	-- be of length >= genSeedLength.  If the bytestring is shorter
-	-- then the call may fail (suggested error: `NotEnoughEntropy`).  If the
-	-- bytestring is of sufficent length the call should always succeed.
+	-- |Instantiate a new random bit generator.  The provided
+	-- bytestring should be of length >= genSeedLength.  If the
+	-- bytestring is shorter then the call may fail (suggested
+	-- error: `NotEnoughEntropy`).  If the bytestring is of
+	-- sufficent length the call should always succeed.
 	newGen :: B.ByteString -> Either GenError g
 
-	-- |Length of input entropy necessary to instantiate or reseed a generator
+	-- |Length of input entropy necessary to instantiate or reseed
+	-- a generator
 	genSeedLength :: Tagged g ByteLength
 
-	-- | @genBytes len g@ generates a random ByteString of length @len@ and new generator.
-	-- The "MonadCryptoRandom" package has routines useful for converting the ByteString
-	-- to commonly needed values (but "cereal" or other deserialization libraries would also work).
+	-- | @genBytes len g@ generates a random ByteString of length
+	-- @len@ and new generator.  The "MonadCryptoRandom" package
+	-- has routines useful for converting the ByteString to
+	-- commonly needed values (but "cereal" or other
+	-- deserialization libraries would also work).
 	--
-	-- This routine can fail if the generator has gone too long without a reseed (usually this
-	-- is in the ball-park of 2^48 requests).  Suggested error in this cases is `NeedReseed`
+	-- This routine can fail if the generator has gone too long
+	-- without a reseed (usually this is in the ball-park of 2^48
+	-- requests).  Suggested error in this cases is `NeedReseed`
 	genBytes	:: ByteLength -> g -> Either GenError (B.ByteString, g)
 
-	-- |@genBytesWithEntropy g i entropy@ generates @i@ random bytes and use the
-	-- additional input @entropy@ in the generation of the requested data to
-	-- increase the confidence our generated data is a secure random stream.
+	-- |@genBytesWithEntropy g i entropy@ generates @i@ random
+	-- bytes and use the additional input @entropy@ in the
+	-- generation of the requested data to increase the confidence
+	-- our generated data is a secure random stream.
 	--
-	-- Some generators use @entropy@ to perturb the state of the generator, meaning:
+	-- Some generators use @entropy@ to perturb the state of the
+	-- generator, meaning:
 	--
 	-- @
 	--     (_,g2') <- genBytesWithEntropy len g1 ent
@@ -104,13 +115,16 @@ class CryptoRandomGen g where
 				let entropy' = B.append entropy (B.replicate (len - B.length entropy) 0)
 				in Right (zwp' entropy' bs, g')
 
-	-- |If the generator has produced too many random bytes on its existing seed
-	-- it will throw `NeedReseed`.  In that case, reseed the generator using this function and
-	-- a new high-entropy seed of length >= `genSeedLength`.  Using bytestrings that are too short
-	-- can result in an error (`NotEnoughEntropy`).
+	-- |If the generator has produced too many random bytes on its
+	-- existing seed it will throw `NeedReseed`.  In that case,
+	-- reseed the generator using this function and a new
+	-- high-entropy seed of length >= `genSeedLength`.  Using
+	-- bytestrings that are too short can result in an error
+	-- (`NotEnoughEntropy`).
 	reseed		:: B.ByteString -> g -> Either GenError g
 
-	-- |By default this uses "System.Crypto.Random" to obtain entropy for `newGen`.
+	-- |By default this uses "System.Crypto.Random" to obtain
+	-- entropy for `newGen`.
 	newGenIO :: IO g
 	newGenIO = go 0
 	  where
@@ -134,8 +148,10 @@ getSystemGen = do
                 return (bs:more)
         liftM (SysRandom . L.fromChunks) getBS
 
--- |Not that it is technically correct as an instance of 'CryptoRandomGen', but simply because
--- it's a reasonable engineering choice here is a CryptoRandomGen which streams the system randoms. Take note:
+-- |Not that it is technically correct as an instance of
+-- 'CryptoRandomGen', but simply because it's a reasonable engineering
+-- choice here is a CryptoRandomGen which streams the system
+-- randoms. Take note:
 -- 
 --  * It uses the default definition of 'genByteWithEntropy'
 --
@@ -144,6 +160,7 @@ getSystemGen = do
 --  * 'reseed' will always fail!
 --
 --  * the handle to the system random is never closed
+--
 data SystemRandom = SysRandom L.ByteString
 
 instance CryptoRandomGen SystemRandom where
@@ -159,9 +176,11 @@ instance CryptoRandomGen SystemRandom where
         reseed _ _ = Left NeedsInfiniteSeed
         newGenIO = getSystemGen
 
--- |While the safety and wisdom of a splitting function depends on the properties of the generator being split,
--- several arguments from informed people indicate such a function is safe for NIST SP 800-90 generators.
--- (see libraries@haskell.org discussion ~ Sept, Oct 2010)
+-- | While the safety and wisdom of a splitting function depends on the
+-- properties of the generator being split, several arguments from
+-- informed people indicate such a function is safe for NIST SP 800-90
+-- generators.  (see libraries\@haskell.org discussion around Sept, Oct
+-- 2010)
 splitGen :: CryptoRandomGen g => g -> Either GenError (g,g)
 splitGen g = do
 	let e = genBytes (genSeedLength `for` g) g
@@ -176,6 +195,7 @@ splitGen g = do
 for :: Tagged a b -> a -> b
 for t _ = unTagged t
 
+-- |Helper function to convert bytestrings to integers
 bs2i :: B.ByteString -> Integer
 bs2i bs = B.foldl' (\i b -> (i `shiftL` 8) + fromIntegral b) 0 bs
 {-# INLINE bs2i #-}
